// $Id: fd_imc3.h,v 1.6 2000/02/17 09:00:15 edwin Exp $

/*
 * Copyright (c) 2000 Fatal Dimensions
 *
 * See the file "LICENSE" or information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

// IMC3 router information
#define IMC3_ROUTER             "216.98.238.194"
#define IMC3_PORT               9000

// IMC3 client information
#define IMC3_THISMUD            "MavEtJu"
#define IMC3_ROUTER_NAME        "*gjs"
#define IMC3_PASSWORD           "0"
#define IMC3_MUDLIST_ID         "10000000"
#define IMC3_CHANLIST_ID        "10000000"
#define IMC3_MUDLIB             "\"Fatal Dimensions\""
#define IMC3_BASE_MUDLIB        "\"Rom24b6\""
#define IMC3_DRIVER             "\"Fatal Dimensions\""
#define IMC3_MUD_TYPE           "\"Diku\""
#define IMC3_OPEN_STATUS        "\"Development\""
#define IMC3_ADMIN_EMAIL        "\"mavetju@chello.nl\""

// internal structures
typedef struct imc3_channel	IMC3_CHANNEL;
typedef struct imc3_listener	IMC3_LISTENER;
typedef struct imc3_mud		IMC3_MUD;
typedef struct imc3_stats	IMC3_STATS;
typedef struct imc3_header	IMC3_HEADER;

struct imc3_listener {
    IMC3_LISTENER *next;
    CHAR_DATA *ch;
};

struct imc3_header {
    char originator_mudname[256];
    char originator_username[256];
    char target_mudname[256];
    char target_username[256];
};

struct imc3_channel {
    IMC3_CHANNEL *next;
    IMC3_LISTENER *listener;
    char *local_name;
    char *host_mud;
    char *imc3_name;
    int status;
    int local_level;
    bool connected;
};

struct imc3_mud {
    IMC3_MUD *next;
    int  status;
    char *name;
    char *ipaddress;
    int  player_port;
    int  imud_tcp_port;
    int  imud_udp_port;
    char *mudlib;
    char *base_mudlib;
    char *driver;
    char *mud_type;
    char *open_status;
    char *admin_email;
};

struct imc3_stats {
    int count_tell;
    int count_emoteto;
    int count_who_commands;
    int count_who_req;
    int count_who_reply;
    int count_finger_commands;
    int count_finger_req;
    int count_finger_reply;
    int count_locate_commands;
    int count_locate_req;
    int count_locate_reply;
    int count_chanlist_reply;
    int count_channel_m_commands;
    int count_channel_m;
    int count_channel_e_commands;
    int count_channel_e;
    int count_channel_t_commands;
    int count_channel_t;
    int count_channel_add;
    int count_channel_remove;
    int count_channel_admin;
    int count_channel_filter_req;
    int count_channel_filter_reply;
    int count_channel_who_commands;
    int count_channel_who_req;
    int count_channel_who_reply;
    int count_channel_listen;
    int count_chan_user_req;
    int count_chan_user_reply;
    int count_news_read_req;
    int count_news_post_req;
    int count_news_grplist_req;
    int count_mail;
    int count_mail_ack;
    int count_file_list_req;
    int count_file_list_reply;
    int count_file_put;
    int count_file_get_req;
    int count_file_get_reply;
    int count_auth_mud_req;
    int count_auth_mud_reply;
    int count_ucache_update;
    int count_error;
    int count_startup_req_3;
    int count_startup_reply;
    int count_shutdown;
    int count_mudlist;
    int count_oob_req;
    int count_oob_begin;
    int count_oob_end;

    int count_unknown;
    int count_total;
};

extern IMC3_CHANNEL *IMC3_channel_list;
extern IMC3_MUD *IMC3_mud_list;
extern IMC3_STATS IMC3_stats;

// fd_imc3_recycle.c
IMC3_CHANNEL *new_imc3_channel(void);
void destroy_imc3_channel(IMC3_CHANNEL *channel);
IMC3_CHANNEL *find_imc3_channel_by_name(char *name);
IMC3_CHANNEL *find_imc3_channel_by_localname(char *name);

IMC3_MUD *new_imc3_mud(void);
void destroy_imc3_mud(IMC3_MUD *mud);
IMC3_MUD *find_imc3_mud_by_name(char *name);

IMC3_LISTENER *find_imc3_listener_by_char(IMC3_CHANNEL *channel,CHAR_DATA *ch);
void destroy_imc3_listener(IMC3_CHANNEL *channel,IMC3_LISTENER *listener);
IMC3_LISTENER *new_imc3_listener(IMC3_CHANNEL *channel);


// fd_imc3_main.c
char *IMC3_get_field(char *packet,char **ps);
//void IMC3_write_packet(char *msg);
void IMC3_write_buffer(char *msg);
void IMC3_send_buffer();
void IMC3_main(void);
void IMC3_loop(void);
void IMC3_shutdown(void);
void IMC3_remove_quotes(char **ps);
char *IMC3_escape(char *ps);
void IMC3_get_header(char **pps,IMC3_HEADER *header);
void IMC3_write_header(char *identifier,char *originator_mudname,
		       char *originator_username,char *target_mudname,
		       char *target_username);
CHAR_DATA *IMC3_find_user(char *name);
bool IMC3_command_hook(CHAR_DATA *ch,char *command,char *argument);
#ifndef HAVE_SPRINTFTOCHAR
void sprintf_to_char(CHAR_DATA *ch, const char *fmt, ...);
#endif

// fd_imc3_tools.c
void imc3_printstats(int i);

// fd_imc3_support.c
int IMC3_process_startup_reply(char *s);
int IMC3_process_mudlist(char *s);
void IMC3_startup_packet(void);
void IMC3_process_error(char *s);

// fd_imc3_channel.c
int IMC3_check_channel(IMC3_CHANNEL *channel);
int IMC3_process_chanlist_reply(char *s);
int IMC3_process_channel_m(char *s);
int IMC3_process_channel_e(char *s);
int IMC3_process_chan_who_req(char *s);
int IMC3_process_chan_who_reply(char *s);
int IMC3_send_channel_listen(IMC3_CHANNEL *channel,bool connect);
int IMC3_send_channel_message(IMC3_CHANNEL *channel,char *name,char *message);
int IMC3_send_channel_emote(IMC3_CHANNEL *channel,char *name,char *message);
int IMC3_send_chan_who(CHAR_DATA *ch,IMC3_CHANNEL *channel,IMC3_MUD *mud);

// fd_imc3_who.c
int IMC3_send_who(CHAR_DATA *ch,char *mud);
int IMC3_process_who_reply(char *s);
int IMC3_process_who_req(char *s);

// fd_imc3_finger.c
int IMC3_send_finger(CHAR_DATA *ch,char *user,char *mud);
int IMC3_process_finger_reply(char *s);
int IMC3_process_finger_req(char *s);

// fd_imc3_locate.c
int IMC3_send_locate(CHAR_DATA *ch,char *user);
int IMC3_process_locate_reply(char *s);
int IMC3_process_locate_req(char *s);
