// $Id: fd_imc3_channel.c,v 1.4 2000/02/17 09:00:16 edwin Exp $

/*
 * Copyright (c) 2000 Fatal Dimensions
 * 
 * See the file "LICENSE" or information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 * 
 */

#include <stdio.h>
#include <time.h>
#include <stdlib.h>
#include "../merc.h"
#include "fd_imc3.h"

// sees if a channel should be connected or disconnected
int IMC3_check_channel(IMC3_CHANNEL *channel) {
    char buf[MSL];

    if (channel->listener==NULL && channel->connected!=0) {
	IMC3_send_channel_listen(channel,FALSE);
	sprintf(buf,"IMC3 - check_channel: unsubscribing from %s (%s @ %s)",
	    channel->local_name,channel->imc3_name,channel->host_mud);
	wiznet(buf,NULL,NULL,WIZ_IMC3_DEBUG,0,0);
	channel->connected=FALSE;
	return 0;
    }
    if (channel->listener!=NULL && channel->connected==0) {
	IMC3_send_channel_listen(channel,TRUE);
	sprintf(buf,"IMC3 - check_channel: subscribing to %s (%s @ %s)",
	    channel->local_name,channel->imc3_name,channel->host_mud);
	wiznet(buf,NULL,NULL,WIZ_IMC3_DEBUG,0,0);
	channel->connected=TRUE;
	return 0;
    }
    return 0;
}

/*
   When a mud sends a startup-req-2 packet, it includes its chanlist-id
   in the packet. The router will potentially respond with a
   chanlist-reply message to update the mud's channel list.           
        
   The router will respond to channel list changes with the                     
   chanlist-reply packet.  
    ({                                                                   
        (string)  "chanlist-reply",                                      
        (int)     5,
        (string)  originator_mudname,     // the router               
        (string)  0,                                                  
        (string)  target_mudname,
        (string)  0,                                                   
        (int)     chanlist_id,                                        
        (mapping) channel_list                                        
    })                     
     
   channel_list is mapping with channel names as keys, and an array of 
   two elements as the values. If the value is 0, then the channel has
   been deleted. The array contains the host mud, and the type of the
   channel:                                                      
        0  selectively banned
        1  selectively admitted                                       
        2  filtered (selectively admitted)                            
   
   All channel messages are delivered to the router. It will then pass   
   the message to the appropriate set of muds. If the channel is         
   filtered, then the packet will be delivered to the host mud for       
   filtering; it will then return to the router network for distribution.
   It is assumed that a channel packet for a filtered channel that comes
   from the channel host has been filtered.                         


*/

int IMC3_process_chanlist_reply(char *s) {
    char *ps=s,*next_ps;
    IMC3_CHANNEL *channel;
    IMC3_HEADER header;

    IMC3_get_header(&ps,&header);
    IMC3_get_field(ps,&next_ps);	// chanlist_id
    ps=next_ps;
    ps+=2;

    while (1) {
	char *next_ps2;

	// get the name of the channel
	IMC3_get_field(ps,&next_ps);
//	printf("Downloading %s\n",ps);

	channel=find_imc3_channel_by_name(ps);
	if (channel==NULL) {
	    channel=new_imc3_channel();
	    free_string(channel->imc3_name);
	    IMC3_remove_quotes(&ps);
	    channel->imc3_name=str_dup(ps);
	}

	ps=next_ps;
	IMC3_get_field(ps,&next_ps2);
	if (ps[0]!='0') {
	    ps+=2;
	    IMC3_get_field(ps,&next_ps);
	    IMC3_remove_quotes(&ps);
	    channel->host_mud=str_dup(ps);
	    ps=next_ps;
	    IMC3_get_field(ps,&next_ps);
	    channel->status=atoi(ps);
	}
	ps=next_ps2;
	if (ps[0]==']')
	    break;
    }


    return 0;
}

/*
   A mud may decide whether or not it is listening to any given channel
   by sending a channel-listen packet. This packet is also used to tune
   out a channel, which should be done whenever no one on the mud is
   listening to the channel. The format of this packet is:
    ({
        (string) "channel-listen",
        (int)    5,                                                    
        (string) originator_mudname,                            
        (string) 0,
        (string) target_mudname,         // the router             
        (string) 0,                                                    
        (string) channel_name,
        (int)    on_or_off
    })                                                                   
     
   The on_or_off will contain one of the following values:               
        0 The mud does not wish to receive this channel.       
        1 The mud wishes to receive this channel.

  ...8({"channel-listen",5,"NoNameMUD",0,"*gjs",0,"test",1,}).          
  ...?({"channel-listen",5,"NoNameMUD",0,"*gjs",0,"imud_gossip",1,}).   
*/
int IMC3_send_channel_listen(IMC3_CHANNEL *channel,bool connect) {

    IMC3_stats.count_channel_listen++;

    IMC3_write_header("channel-listen",IMC3_THISMUD,NULL,IMC3_ROUTER_NAME,NULL);
    IMC3_write_buffer("\"");
    IMC3_write_buffer(channel->imc3_name);
    IMC3_write_buffer("\",");
    if (connect)
	IMC3_write_buffer("1,})\r");
    else
	IMC3_write_buffer("0,})\r");
    IMC3_send_buffer();
    return 0;
}

/*
   Channel messages come in three flavors: standard messages, emotes, and
   targetted emotes. These use packets channel-m, channel-e, and
   channel-t, respectively. They are:                                   
    ({                                                                  
        (string) "channel-m",                                                   
        (int)    5,
        (string) originator_mudname,
        (string) originator_username,
        (string) 0,                                                     
        (string) 0,                                                  
        (string) channel_name,                                   
        (string) visname,                                                       
        (string) message
    })                                                                
*/
int IMC3_send_channel_message(IMC3_CHANNEL *channel,char *name,char *message) {

    IMC3_stats.count_channel_m_commands++;

    IMC3_write_header("channel-m",IMC3_THISMUD,name,NULL,NULL);
    IMC3_write_buffer("\"");
    IMC3_write_buffer(channel->imc3_name);
    IMC3_write_buffer("\",\"");
    IMC3_write_buffer(name);
    IMC3_write_buffer("\",\"");
    IMC3_write_buffer(IMC3_escape(message));
    IMC3_write_buffer("\",})\r");
    IMC3_send_buffer();

    return 0;
}

/*
   Channel messages come in three flavors: standard messages, emotes, and
   targetted emotes. These use packets channel-m, channel-e, and
   channel-t, respectively. They are:                                   
    ({                                                                  
        (string) "channel-e",                                                   
        (int)    5,
        (string) originator_mudname,
        (string) originator_username,
        (string) 0,                                                     
        (string) 0,                                                  
        (string) channel_name,                                   
        (string) visname,                                                       
        (string) message
    })                                                                
*/
int IMC3_send_channel_emote(IMC3_CHANNEL *channel,char *name,char *message) {

    IMC3_stats.count_channel_e_commands++;

    IMC3_write_header("channel-e",IMC3_THISMUD,name,NULL,NULL);
    IMC3_write_buffer("\"");
    IMC3_write_buffer(channel->imc3_name);
    IMC3_write_buffer("\",\"");
    IMC3_write_buffer(name);
    IMC3_write_buffer("\",\"");
    IMC3_write_buffer(IMC3_escape(message));
    IMC3_write_buffer("\",})\r");
    IMC3_send_buffer();

    return 0;
}

int IMC3_process_channel_m(char *s) {
    char *ps=s,*next_ps;
    IMC3_HEADER header;
    char visname[MSL];
    char message[MSL];
    char buf[MSL];
    IMC3_CHANNEL *channel;
    IMC3_LISTENER *listener;

    IMC3_get_header(&ps,&header);

    IMC3_get_field(ps,&next_ps);
    IMC3_remove_quotes(&ps);
    if ((channel=find_imc3_channel_by_name(ps))==NULL) {
	sprintf(buf,"IMC3 - channel_m: received unknown channel (%s)",ps);
	wiznet(buf,NULL,NULL,WIZ_IMC3_ERRORS,0,0);
	return 0;
    }

    ps=next_ps;
    IMC3_get_field(ps,&next_ps);
    IMC3_remove_quotes(&ps);
    strcpy(visname,ps);

    ps=next_ps;
    IMC3_get_field(ps,&next_ps);
    IMC3_remove_quotes(&ps);
    strcpy(message,ps);

    sprintf(buf,"[%s] %s@%s: %s\n\r",
	channel->local_name,visname,header.originator_mudname,message);
    for (listener=channel->listener;listener;listener=listener->next) {
	send_to_char(buf,listener->ch);
    }

    return 0;
}


int IMC3_process_channel_e(char *s) {
    char *ps=s,*next_ps;
    IMC3_HEADER header;
    char visname[MSL];
    char message[MSL];
    char buf[MSL];
    IMC3_CHANNEL *channel;
    IMC3_LISTENER *listener;

    IMC3_get_header(&ps,&header);

    IMC3_get_field(ps,&next_ps);
    IMC3_remove_quotes(&ps);
    if ((channel=find_imc3_channel_by_name(ps))==NULL) {
	sprintf(buf,"IMC3 - channel_e: received unknown channel (%s)",ps);
	wiznet(buf,NULL,NULL,WIZ_IMC3_ERRORS,0,0);
	return 0;
    }

    ps=next_ps;
    IMC3_get_field(ps,&next_ps);
    IMC3_remove_quotes(&ps);
    strcpy(visname,ps);

    ps=next_ps;
    IMC3_get_field(ps,&next_ps);
    IMC3_remove_quotes(&ps);
    strcpy(message,ps);

    sprintf(buf,"[%s] %s@%s %s\n\r",
	channel->local_name,visname,header.originator_mudname,message);
    for (listener=channel->listener;listener;listener=listener->next) {
	send_to_char(buf,listener->ch);
    }

    return 0;
}

/*
   A list of who is listening to a channel on a remote mud may be
   requested with the following packet:            
    ({                                                                   
        (string) "chan-who-req",                      
        (int)    5,
        (string) originator_mudname,
        (string) originator_username,
        (string) target_mudname,
        (string) 0,
        (string) channel_name
    })                                                              
*/
int IMC3_process_chan_who_req(char *s) {
    IMC3_HEADER header;
    char *ps=s,*next_ps;
    IMC3_CHANNEL *channel;
    IMC3_LISTENER *listener;

    IMC3_get_header(&ps,&header);
    IMC3_get_field(ps,&next_ps);
    IMC3_remove_quotes(&ps);
    if ((channel=find_imc3_channel_by_name(ps))==NULL) {
	char buf[MSL];

	sprintf(buf,"IMC3 - chan_who_req: received unknown channel (%s)",ps);
	wiznet(buf,NULL,NULL,WIZ_IMC3_ERRORS,0,0);
	return 0;
    }

    IMC3_write_header("chan-who-reply",IMC3_THISMUD,NULL,
	header.originator_mudname,header.originator_username);
    IMC3_write_buffer("\"");
    IMC3_write_buffer(channel->imc3_name);
    IMC3_write_buffer("\",({");

    for (listener=channel->listener;listener;listener=listener->next) {
	IMC3_write_buffer("\"");
	IMC3_write_buffer(listener->ch->name);
	IMC3_write_buffer("\",");
    }
    IMC3_write_buffer("}),})\r");
    IMC3_send_buffer();

    return 0;
}

/*
   The reply for the who request takes the following format:
    ({                                                               
        (string)   "chan-who-reply",
        (int)      5,
        (string)   originator_mudname,
        (string)   0,
        (string)   target_mudname,
        (string)   target_username, 
        (string)   channel_name,                      
        (string *) user_list                   
    })                       
   
   The user_list should be an array of strings, representing the users'
   "visual" names.                                                 
*/
int IMC3_process_chan_who_reply(char *s) {
    char *ps=s,*next_ps;
    IMC3_HEADER header;
    CHAR_DATA *ch;

    IMC3_get_header(&ps,&header);
    if ((ch=IMC3_find_user(header.target_username))==NULL) {
	char s[MSL];

	sprintf(s,"IMC3 - IMC3_process_chan_who_reply(): user %s not found.",
	    header.target_username);
	bug(s,0);
	return 0;
    }

    IMC3_get_field(ps,&next_ps);
    IMC3_remove_quotes(&ps);
    sprintf_to_char(ch,"CHANWHO reply from %s for %s\n\r",
	header.originator_mudname,ps);

    ps=next_ps;
    IMC3_get_field(ps,&next_ps);
    ps+=2;
    while (1) {
	if (ps[0]=='}') {
	    send_to_char(
		"No information returned or no people listening.\n\r",ch);
	    return 0;
	}

	IMC3_get_field(ps,&next_ps);
	IMC3_remove_quotes(&ps);
	sprintf_to_char(ch,"- %s\n\r",ps);

	ps=next_ps;
	if (ps[0]=='}')
	    break;
    }

    return 0;
}

int IMC3_send_chan_who(CHAR_DATA *ch,IMC3_CHANNEL *channel,IMC3_MUD *mud) {

    IMC3_stats.count_channel_who_commands++;

    IMC3_write_header("chan-who-req",IMC3_THISMUD,ch->name,mud->name,NULL);
    IMC3_write_buffer("\"");
    IMC3_write_buffer(channel->imc3_name);
    IMC3_write_buffer("\",})\r");
    IMC3_send_buffer();

    return 0;
}

