// $Id: fd_imc3_finger.c,v 1.5 2000/02/17 09:00:16 edwin Exp $

/*
 * Copyright (c) 2000 Fatal Dimensions
 * 
 * See the file "LICENSE" or information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 * 
 */

#include <stdio.h>
#include <time.h>
#include <strings.h>
#include "../merc.h"
#ifdef IS_FD
#include "../property.h"
#endif
#include "fd_imc3.h"

int IMC3_send_finger(CHAR_DATA *ch,char *user,char *mud) {

    IMC3_stats.count_finger_commands++;

    IMC3_escape(mud);
    IMC3_escape(user);

    IMC3_write_header("finger-req",IMC3_THISMUD,ch->name,mud,NULL);
    IMC3_write_buffer("\"");
    IMC3_write_buffer(user);
    IMC3_write_buffer("\",})\r");
    IMC3_send_buffer();
    return 0;
}

/*
   The target mud will return:
    ({
        (string) "finger-reply",     
        (int)    5,
        (string) originator_mudname, 
        (string) 0,
        (string) target_mudname,
        (string) target_username,
        (string) visname,
        (string) title,
        (string) real_name,
        (string) e_mail,
        (string) loginout_time,
        (int)    idle_time,
        (string) ip_name,     
        (string) level,             
        (string) extra  // eg, a .plan file, or other info
    })

   A mud may return 0 for any item if they wish to keep the information
   private. In particular, it is suggested that information about players
   (as opposed to wizards) be kept confidential.

   The returned visname should contain the user's visual name.
   loginout_time specifies the (local) time the user logged in (if they
   are currently on) or the time the user logged out. The value should be
   expressed as a string. It should be 0 to indicate no information. The
   idle_time is expressed as an integer number of seconds of idle time.  
   If this value is -1, then the user is not logged onto the mud at the
   moment.

   If extra is given, then it should be terminated with a carriage
   return.                                                           

({"finger-reply",5,"NoNameMUD",0,"MavEtJu","MavEtJu","Mavetju","Mavetju the Newbie","Mavetju","[Private]","Sat Feb 12 22:47:44 2000",45,0,"Player",0,})
*/

int IMC3_process_finger_reply(char *s) {
    IMC3_HEADER header;
    CHAR_DATA *ch;
    char *ps=s,*next_ps;

    IMC3_get_header(&ps,&header);
    if ((ch=IMC3_find_user(header.target_username))==NULL) {
	char s[MSL];
	sprintf(s,"IMC3 - IMC3_process_who_reply(): user %s not found.",
	    header.target_username);
	bug(s,0);
	return 0;
    }

    IMC3_get_field(ps,&next_ps);
    IMC3_remove_quotes(&ps);
    sprintf_to_char(ch,"FINGER information for %s at %s\n\r",
	ps,header.originator_mudname);
    ps=next_ps;

    IMC3_get_field(ps,&next_ps);
    IMC3_remove_quotes(&ps);
    sprintf_to_char(ch,"Title     : %s\n\r",ps);
    ps=next_ps;

    IMC3_get_field(ps,&next_ps);
    IMC3_remove_quotes(&ps);
    sprintf_to_char(ch,"Real name : %s\n\r",ps);
    ps=next_ps;

    IMC3_get_field(ps,&next_ps);
    IMC3_remove_quotes(&ps);
    sprintf_to_char(ch,"Email     : %s\n\r",ps);
    ps=next_ps;

    IMC3_get_field(ps,&next_ps);
    IMC3_remove_quotes(&ps);
    sprintf_to_char(ch,"Login at  : %s\n\r",ps);
    ps=next_ps;

    IMC3_get_field(ps,&next_ps);
    sprintf_to_char(ch,"Idle for  : %s seconds\n\r",ps);
    ps=next_ps;

    IMC3_get_field(ps,&next_ps);
    IMC3_remove_quotes(&ps);
    sprintf_to_char(ch,"IP address: %s\n\r",ps);
    ps=next_ps;

    IMC3_get_field(ps,&next_ps);
    IMC3_remove_quotes(&ps);
    sprintf_to_char(ch,"Level     : %s\n\r",ps);
    ps=next_ps;

    IMC3_get_field(ps,&next_ps);
    IMC3_remove_quotes(&ps);
    sprintf_to_char(ch,"Other info: %s\n\r",ps);

    return 0;
}

/*
    ({
        (string) "finger-req",       
        (int)    5,
        (string) originator_mudname, 
        (string) originator_username,
        (string) target_mudname,
        (string) 0,   
        (string) username
    })

*/
int IMC3_process_finger_req(char *s) {
    IMC3_HEADER header;
    CHAR_DATA *ch;
    char *ps=s,*next_ps;
    char buf[MSL];
    char smallbuf[MSL];

    IMC3_get_header(&ps,&header);
    IMC3_get_field(ps,&next_ps);
    IMC3_remove_quotes(&ps);
    if ((ch=IMC3_find_user(ps))==NULL) {
	// perhaps we should be nice to the person...
	sprintf(buf,"IMC3 - Finger: failed request from %s @ %s for %s.",
	    header.originator_username,header.originator_mudname,ps);
	wiznet(buf,NULL,NULL,WIZ_IMC3_INFO,0,0);
	return 0;
    }

    sprintf(buf,"IMC3 - Finger: request from %s @ %s for %s.",
	header.originator_username,header.originator_mudname,
	ch->name);
    wiznet(buf,NULL,NULL,WIZ_IMC3_INFO,0,0);
    sprintf_to_char(ch,"You have been rfingered by %s @ %s\n\r",
	header.originator_username,header.originator_mudname);

    IMC3_write_header("finger-reply",IMC3_THISMUD,NULL,
	header.originator_mudname,header.originator_username);
    IMC3_write_buffer("\"");
    IMC3_write_buffer(IMC3_escape(ch->name));
    IMC3_write_buffer("\",\"");
    IMC3_write_buffer(IMC3_escape(ch->name));
    IMC3_write_buffer(IMC3_escape(ch->pcdata->title));
    IMC3_write_buffer("\",\"\",\"");			// real name
#ifdef IS_FD
    if (GetCharProperty(ch,PROPERTY_STRING,"email",smallbuf)) {
#else
    if (0) {
#endif
	IMC3_write_buffer(smallbuf);			// email address
    } else {
	IMC3_write_buffer("none");			// email address
    }
    IMC3_write_buffer("\",\"");
    IMC3_write_buffer(ctime_r(&ch->logon,smallbuf));	// online since
    IMC3_write_buffer("\",");
#ifdef IS_FD
    sprintf(smallbuf,"%ld",time(NULL)-ch->pcdata->idle);
#else
    sprintf(smallbuf,"%ld",-1l);
#endif
    IMC3_write_buffer(smallbuf);			// idle since
    IMC3_write_buffer(",\"");
    IMC3_write_buffer("[PRIVATE]");			// IP address
    IMC3_write_buffer("\",\"");
#ifdef IS_FD
    IMC3_write_buffer(ch->pcdata->whoname[0]?
		IMC3_escape(ch->pcdata->whoname):"Player");
						// Level
#else
    IMC3_write_buffer("Player");
#endif
    IMC3_write_buffer("\",\"none\",})\r");		// No extra info
    IMC3_send_buffer();
    return 0;
}

