// $Id: fd_imc3_locate.c,v 1.4 2000/02/17 09:00:17 edwin Exp $

/*
 * Copyright (c) 2000 Fatal Dimensions
 * 
 * See the file "LICENSE" or information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 * 
 */

#include <stdio.h>
#include <time.h>
#include <strings.h>
#include "../merc.h"
#include "fd_imc3.h"

int IMC3_send_locate(CHAR_DATA *ch,char *user) {

    IMC3_stats.count_locate_commands++;

    IMC3_escape(user);
    IMC3_write_header("locate-req",IMC3_THISMUD,ch->name,NULL,NULL);
    IMC3_write_buffer("\"");
    IMC3_write_buffer(user);
    IMC3_write_buffer("\",})");
    IMC3_send_buffer();
    return 0;
}

/*
   If the requested user is logged into the receiving mud, then the
   following reply is returned:
    ({
        (string) "locate-reply",     
        (int)    5,
        (string) originator_mudname, 
        (string) 0,
        (string) target_mudname,
        (string) target_username,
        (string) located_mudname,
        (string) located_visname,
        (int)    idle_time,
        (string) status,
    })   

   located_visname should contain the correct visual name of the user.
   idle_time will have the idle time (in seconds) of the located user.

   status specifies any special status of the user. This will typically
   be zero to indicate that the user has no special status. The values
   for this string are arbitrary, but certain statuses have predefined
   values that can be used:                                      
     * "link-dead"                                      
     * "editing"                                                      
     * "inactive"
     * "invisible"                                                     
     * "hidden"                                                 

({"locate-reply",5,"NoNameMUD",0,"MavEtJu","MavEtJu","NoNameMUD","Mavetju",277,"active",})
*/

int IMC3_process_locate_reply(char *s) {
    char *ps=s,*next_ps;
    CHAR_DATA *ch;
    IMC3_HEADER header;

    IMC3_get_header(&ps,&header);

    if ((ch=IMC3_find_user(header.target_username))==NULL) {
	char s[MSL];
	sprintf(s,"IMC3 - IMC3_process_locate_reply(): user %s not found.",
	    header.target_username);
	bug(s,0);
	return 0;
    }

    IMC3_get_field(ps,&next_ps);
    IMC3_remove_quotes(&ps);
    sprintf_to_char(ch,"RLOCATE reply from %s\n\r",ps);
    ps=next_ps;

    IMC3_get_field(ps,&next_ps);
    IMC3_remove_quotes(&ps);
    sprintf_to_char(ch,"Username  : %s\n\r",ps);
    ps=next_ps;

    IMC3_get_field(ps,&next_ps);
    sprintf_to_char(ch,"Idle since: %s seconds\n\r",ps);
    ps=next_ps;

    IMC3_get_field(ps,&next_ps);
    IMC3_remove_quotes(&ps);
    sprintf_to_char(ch,"Status    : %s\n\r",ps);

    return 0;
}

int IMC3_process_locate_req(char *s) {
    char *ps=s,*next_ps;
    IMC3_HEADER header;
    char buf[MSL],smallbuf[MSL];
    CHAR_DATA *ch;

    IMC3_get_header(&ps,&header);
    IMC3_get_field(ps,&next_ps);
    IMC3_remove_quotes(&ps);
    sprintf(buf,"IMC3 - Locate: request from %s @ %s for %s.",
	header.originator_username,header.originator_mudname,ps);
    wiznet(buf,NULL,NULL,WIZ_IMC3_INFO,0,0);

    if ((ch=IMC3_find_user(ps))==NULL) {
	return 0;
    }

    IMC3_write_header("locate-reply",IMC3_THISMUD,NULL,
		      header.originator_mudname,header.originator_username);
    IMC3_write_buffer("\"");
    IMC3_write_buffer(IMC3_THISMUD);
    IMC3_write_buffer("\",\"");
    IMC3_write_buffer(ch->name);
    IMC3_write_buffer("\",");
#ifdef IS_FD
    sprintf(smallbuf,"%ld",time(NULL)-ch->pcdata->idle);
#else
    sprintf(smallbuf,"%ld",-1l);
#endif
    IMC3_write_buffer(smallbuf);
    IMC3_write_buffer(",\"active\",})");
    IMC3_send_buffer();

    return 0;
}

