// $Id: fd_imc3_main.c,v 1.7 2000/02/17 11:24:42 edwin Exp $

/*
 * Copyright (c) 2000 Fatal Dimensions
 * 
 * See the file "LICENSE" or information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 * 
 */

#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <sys/time.h>
#include <unistd.h>
#include <stdlib.h>
#include <signal.h>
#include <stdarg.h>
#include <ctype.h>
#include "../merc.h"
#include "fd_imc3.h"

char IMC3_input_buffer[256*256];
char IMC3_output_buffer[256*256];
long IMC3_input_pointer=0;
long IMC3_output_pointer=0;
char *IMC3_currentpacket=NULL;
int  IMC3_socket=0;
IMC3_STATS IMC3_stats;

int IMC3_connection_open(char *host, int port) {
    struct sockaddr_in s_addr;
    struct hostent *hostp;
    u_long ip_addr;
    char s[MSL];

    sprintf(s,"Attempting connect to %s on port %d", host, port);
    log_string(s);

    IMC3_socket = socket(AF_INET, SOCK_STREAM, 0);
    if (IMC3_socket < 0) {
	bug("Cannot create socket",0);
	return -1;
    }

    memset(&s_addr, 0, sizeof(s_addr));
    s_addr.sin_family = AF_INET;
    if ((ip_addr = inet_addr(host)) != INADDR_NONE) {
	memcpy(&s_addr.sin_addr, &ip_addr, sizeof(ip_addr));
    } else {
	hostp = gethostbyname(host);
	if (!hostp) {
	    bug("Cannot resolve hostname",0);
	    return -1;       
	}
	memcpy(&s_addr.sin_addr, hostp->h_addr, hostp->h_length);
    }
    s_addr.sin_port = htons(port);  
    if (connect(IMC3_socket, (struct sockaddr *)&s_addr, sizeof(s_addr)) < 0) {
	bug("Cannot connect to router",0);
	exit(-1);
    }
    log_string("Connected to router.");
    return IMC3_socket;
}

void IMC3_connection_close(void) {
    close(IMC3_socket);
}

void IMC3_write_buffer(char *msg) {
    long newsize=IMC3_output_pointer+strlen(msg);

    if (newsize>256*256-1) {
	bug("IMC3 - IMC3_write_buffer: buffer too large (would become %d)",newsize);
	return;
    }
    strcpy(IMC3_output_buffer+IMC3_output_pointer,msg);
//  printf("Buffering: %ld %ld >%s<\n",IMC3_output_pointer,newsize,IMC3_output);
    IMC3_output_pointer=newsize;
}

void IMC3_write_packet(char *msg) {
    int oldsize,size;
    char s[10];

    oldsize=size=strlen(msg);
    s[3]=size%256;size>>=8;
    s[2]=size%256;size>>=8;
    s[1]=size%256;size>>=8;
    s[0]=size%256;
    write(IMC3_socket,s,4);
    write(IMC3_socket,msg,oldsize);
}

void IMC3_send_buffer() {
//  printf("Sending: >%s<\n",IMC3_output);
    IMC3_write_packet(IMC3_output_buffer);
    IMC3_output_pointer=0;
}


int IMC3_local_loop(void) {
    int ret;
    long size;
    fd_set in_set,out_set,exc_set;
    static struct timeval null_time;

    FD_ZERO( &in_set  );
    FD_ZERO( &out_set );
    FD_ZERO( &exc_set );
    FD_SET( IMC3_socket , &in_set );
    if (select(IMC3_socket+1, &in_set, &out_set, &exc_set, &null_time ) < 0 ) {
	perror( "Game_loop: select: poll" );
	exit( 1 );
    }
    if (!FD_ISSET(IMC3_socket,&in_set)) {
	if (IMC3_input_pointer==0)	// perhaps we have enough now
	    return 0;
	ret=0;
    } else {
	ret=read(IMC3_socket,IMC3_input_buffer+IMC3_input_pointer,MSL);
	if (ret<=0)
	    return -1;
	IMC3_input_pointer+=ret;
    }
    memcpy(&size,IMC3_input_buffer,4);
    size=ntohl(size);
//    printf("Read %d bytes, have %d bytes, need %ld bytes\n",
//	ret,IMC3_input_pointer-4,size);
    if (size<=IMC3_input_pointer-4)
	return 1;
    return 0;
}

int IMC3_read_packet(void) {
    long size;

    memcpy(&size,IMC3_input_buffer,4);
    size=ntohl(size);
//  printf("read_packet, %ld bytes\n",size);
    if (IMC3_currentpacket)
	free(IMC3_currentpacket);
    IMC3_currentpacket=(char *)malloc(size+10);
    memcpy(IMC3_currentpacket,IMC3_input_buffer+4,size);
    IMC3_currentpacket[size+1]=0;
//  printf("got %s\n",IMC3_currentpacket);
    memcpy(IMC3_input_buffer,IMC3_input_buffer+size+4,IMC3_input_pointer-size-4);
    IMC3_input_pointer-=size+4;
    return 1;
}

// returns offset of next field
char *IMC3_get_field(char *packet,char **ps) {
    int count[256];
    char has_apostrophe=0,has_backslash=0;
    char foundit=0;

    bzero(count,sizeof(count));

    *ps=packet;
    while (1) {
	switch (*ps[0]) {
	    case '{': if (!has_apostrophe) count['{']++;break;
	    case '}': if (!has_apostrophe) count['}']++;break;
	    case '[': if (!has_apostrophe) count['[']++;break;
	    case ']': if (!has_apostrophe) count[']']++;break;
	    case '(': if (!has_apostrophe) count['(']++;break;
	    case ')': if (!has_apostrophe) count[')']++;break;
	    case '\\':
		if (has_backslash)
		    has_backslash=0;
		else
		    has_backslash=1;
		break;
	    case '"':
		if (has_backslash) {
		    has_backslash=0;
		} else {
		    if (has_apostrophe)
			has_apostrophe=0;
		    else
			has_apostrophe=1;
		}
		break;
	    case ',':
	    case ':':
		if (has_apostrophe)
		    break;
		if (has_backslash)
		    break;
		if (count['{']!=count['}'])
		    break;
		if (count['[']!=count[']'])
		    break;
		if (count['(']!=count[')'])
		    break;
		foundit=1;
		break;
	}
	if (foundit)
	    break;
	(*ps)++;
    }
    *ps[0]=0;
    (*ps)++;
    return *ps;
}


int IMC3_parse_packet(void) {
    char *ps,*next_ps;

    ps=IMC3_currentpacket;
    if ( ps[0]!='(' || ps[1]!='{' ) {
	return 1;
    }
    ps+=2;
    IMC3_get_field(ps,&next_ps);

    IMC3_stats.count_total++;

    if (!strcmp(ps,"\"mudlist\"")) {
	IMC3_stats.count_mudlist++;
	IMC3_process_mudlist(next_ps);
	return 0;
    }
    if (!strcmp(ps,"\"channel-m\"")) {
	IMC3_stats.count_channel_m++;
	IMC3_process_channel_m(next_ps);
	return 0;
    }
    if (!strcmp(ps,"\"channel-e\"")) {
	IMC3_stats.count_channel_e++;
	IMC3_process_channel_e(next_ps);
	return 0;
    }
    if (!strcmp(ps,"\"finger-req\"")) {
	IMC3_stats.count_finger_req++;
	IMC3_process_finger_req(next_ps);
	return 0;
    }
    if (!strcmp(ps,"\"finger-reply\"")) {
	IMC3_stats.count_finger_reply++;
	IMC3_process_finger_reply(next_ps);
	return 0;
    }
    if (!strcmp(ps,"\"locate-req\"")) {
	IMC3_stats.count_locate_req++;
	IMC3_process_locate_req(next_ps);
	return 0;
    }
    if (!strcmp(ps,"\"locate-reply\"")) {
	IMC3_stats.count_locate_reply++;
	IMC3_process_locate_reply(next_ps);
	return 0;
    }
    if (!strcmp(ps,"\"chan-who-req\"")) {
	IMC3_stats.count_channel_who_req++;
	IMC3_process_chan_who_req(next_ps);
	return 0;
    }
    if (!strcmp(ps,"\"chan-who-reply\"")) {
	IMC3_stats.count_channel_who_reply++;
	IMC3_process_chan_who_reply(next_ps);
	return 0;
    }
    if (!strcmp(ps,"\"who-req\"")) {
	IMC3_stats.count_who_req++;
	IMC3_process_who_req(next_ps);
	return 0;
    }
    if (!strcmp(ps,"\"who-reply\"")) {
	IMC3_stats.count_who_reply++;
	IMC3_process_who_reply(next_ps);
	return 0;
    }
    if (!strcmp(ps,"\"chanlist-reply\"")) {
	IMC3_stats.count_chanlist_reply++;
	IMC3_process_chanlist_reply(next_ps);
	return 0;
    }
    if (!strcmp(ps,"\"startup-reply\"")) {
	IMC3_stats.count_startup_reply++;
	IMC3_process_startup_reply(next_ps);
	return 0;
    }
    if (!strcmp(ps,"\"error\"")) {
	IMC3_stats.count_error++;
	IMC3_process_error(next_ps);
	return 0;
    }

    IMC3_stats.count_unknown++;
    printf("got %s\n",IMC3_currentpacket);
    printf("Found unknown field: .%s.\n",ps);
    return 1;
}


void IMC3_main(void) {

    if ((IMC3_socket=IMC3_connection_open(IMC3_ROUTER,IMC3_PORT))==-1) {
	log_string("Couldn't connect to router.");
	return;
    }

    sleep(1);
//  signal(SIGINFO,imc3_printstats);		// ^T

    IMC3_startup_packet();
}

void IMC3_loop(void) {
    int ret;

    if (IMC3_socket==0)
	return;

    ret=IMC3_local_loop();
    if (ret<0)
	return;
    if (ret>0) {
	IMC3_read_packet();
	IMC3_parse_packet();
    }
}

void IMC3_shutdown(void) {
    if (IMC3_socket==0)
	return;

    log_string("Closing connection to router.");
    IMC3_connection_close();
}

void IMC3_remove_quotes(char **ps) {
    if (*ps[0]=='"')
	(*ps)++;
    if ((*ps)[strlen(*ps)-1]=='"')
	(*ps)[strlen(*ps)-1]=0;
}

char *IMC3_escape(char *ps) {
    static char new[MSL];
    char *pnew=new;

    while (ps[0]) {
	if (ps[0]=='"') {
	    pnew[0]='\\';
	    pnew++;
	}
	pnew[0]=ps[0];
	pnew++;
	ps++;
    }
    pnew[0]=0;
    return new;
}

CHAR_DATA *IMC3_find_user(char *name) {
    CHAR_DATA *ch=NULL;
    DESCRIPTOR_DATA *d;

    for ( d = descriptor_list; d ; d = d->next ) {
	if (d->character &&
	    d->character->name &&
	    tolower(d->character->name[0])==tolower(name[0]) &&
	    !str_cmp(d->character->name,name)) {
	    ch=d->character;
	    break;
	}
    }
    return ch;
}

void IMC3_write_header(char *identifier,char *originator_mudname,
		       char *originator_username,char *target_mudname,
		       char *target_username) {
    IMC3_write_buffer("({\"");
    IMC3_write_buffer(identifier);
    IMC3_write_buffer("\",5,");
    if (originator_mudname) {
	IMC3_write_buffer("\"");
	IMC3_write_buffer(originator_mudname);
	IMC3_write_buffer("\",");
    } else IMC3_write_buffer("0,");
    if (originator_username) {
	IMC3_write_buffer("\"");
	IMC3_write_buffer(originator_username);
	IMC3_write_buffer("\",");
    } else IMC3_write_buffer("0,");
    if (target_mudname) {
	IMC3_write_buffer("\"");
	IMC3_write_buffer(target_mudname);
	IMC3_write_buffer("\",");
    } else IMC3_write_buffer("0,");
    if (target_username) {
	IMC3_write_buffer("\"");
	IMC3_write_buffer(target_username);
	IMC3_write_buffer("\",");
    } else IMC3_write_buffer("0,");
}


void IMC3_get_header(char **pps,IMC3_HEADER *header) {
    char *ps=*pps,*next_ps;

    header->originator_mudname[0]=0;	header->originator_mudname[255]=0;
    header->originator_username[0]=0;	header->originator_username[255]=0;
    header->target_mudname[0]=0;	header->target_mudname[255]=0;
    header->target_username[0]=0;	header->target_username[255]=0;

    IMC3_get_field(ps,&next_ps);
    ps=next_ps;
    IMC3_get_field(ps,&next_ps);
    IMC3_remove_quotes(&ps);
    strncpy(header->originator_mudname,ps,254);
    ps=next_ps;
    IMC3_get_field(ps,&next_ps);
    IMC3_remove_quotes(&ps);
    strncpy(header->originator_username,ps,254);
    ps=next_ps;
    IMC3_get_field(ps,&next_ps);
    IMC3_remove_quotes(&ps);
    strncpy(header->target_mudname,ps,254);
    ps=next_ps;
    IMC3_get_field(ps,&next_ps);
    IMC3_remove_quotes(&ps);
    strncpy(header->target_username,ps,254);
    *pps=next_ps;
}

bool IMC3_command_hook(CHAR_DATA *ch,char *command,char *argument) {
    IMC3_CHANNEL *channel;

    for (channel=IMC3_channel_list;channel;channel=channel->next) {
	if (tolower(channel->local_name[0])==tolower(command[0]) &&
	    !str_cmp(command,channel->local_name)) {
	    switch (argument[0]) {
	    case ',':
		IMC3_send_channel_emote(channel,ch->name,argument);
		break;
	    default:
		IMC3_send_channel_message(channel,ch->name,argument);
		break;
	    }
	    return 1;
	}
    }
    return 0;
}


#ifndef HAVE_SPRINTFTOCHAR
void sprintf_to_char(CHAR_DATA *ch, const char *fmt, ...)
{
    char buf[MAX_STRING_LENGTH];
    va_list ap;

    va_start(ap,fmt);
    vsprintf(buf,fmt,ap);
    va_end(ap);
    send_to_char(buf,ch);
}
#endif

#ifndef HAVE_WHICHKEYWORD
int which_keyword(char *keyword, ...) {
    va_list ap;
    int i=1;
    char *arg;
    char k[MAX_STRING_LENGTH];

    if (!keyword || !keyword[0])
        return -1;

    va_start(ap,keyword);
    keyword=one_argument(keyword,k);
    do {
	arg=va_arg(ap, char *);
	if (arg) {
	    if (!str_prefix(k,arg)) {
		va_end(ap);
		return i;
	    }
	}
        i++;
    } while(arg);
    va_end(ap);
    return 0;
}
#endif
