// $Id: fd_imc3_recycle.c,v 1.3 2000/02/17 09:00:18 edwin Exp $

/*
 * Copyright (c) 2000 Fatal Dimensions
 * 
 * See the file "LICENSE" or information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 * 
 */

#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include "../merc.h"
#include "fd_imc3.h"

IMC3_CHANNEL *IMC3_channel_list=NULL;
IMC3_MUD *IMC3_mud_list=NULL;

IMC3_CHANNEL *find_imc3_channel_by_localname(char *name) {
    IMC3_CHANNEL *this=IMC3_channel_list;
    while (this) {
	if (str_cmp(this->local_name,name)==0)
	    break;
	this=this->next;
    }
    return this;
}

IMC3_CHANNEL *find_imc3_channel_by_name(char *name) {
    IMC3_CHANNEL *this=IMC3_channel_list;
    while (this) {
	if (str_cmp(this->imc3_name,name)==0)
	    break;
	this=this->next;
    }
    return this;
}

IMC3_CHANNEL *new_imc3_channel(void) {
    IMC3_CHANNEL *new;

    new=(IMC3_CHANNEL *)malloc(sizeof(*new));
    bzero(new,sizeof(IMC3_CHANNEL));
    new->next=IMC3_channel_list;
    IMC3_channel_list=new;
    new->local_name=str_dup("");
    new->host_mud=str_dup("");
    new->imc3_name=str_dup("");
    return new;
}

void destroy_imc3_channel(IMC3_CHANNEL *channel) {
    if (channel==NULL) {
	bug("destroy_imc3_channel: Null parameter",0);
	return;
    }

    if (channel==IMC3_channel_list) {
	IMC3_channel_list=IMC3_channel_list->next;
    } else {
	IMC3_CHANNEL *temp=IMC3_channel_list->next;
	while (temp!=channel && temp)
	    temp=temp->next;
	if (temp==NULL) {
	    bug("destroy_imc3_channel: Channel not found in list",0);
	    return;
	}
	temp->next=channel->next;
    }
    free_string(channel->local_name);
    free_string(channel->imc3_name);
    free_string(channel->host_mud);

    while (channel->listener)
	destroy_imc3_listener(channel,channel->listener);
}

// IMC3_MUD functions

IMC3_MUD *find_imc3_mud_by_name(char *name) {
    IMC3_MUD *this=IMC3_mud_list;
    while (this) {
	if (str_cmp(this->name,name)==0)
	    break;
	this=this->next;
    }
    return this;
}

IMC3_MUD *new_imc3_mud(void) {
    IMC3_MUD *new;

    new=(IMC3_MUD *)malloc(sizeof(IMC3_MUD));
    bzero(new,sizeof(IMC3_MUD));
    new->next=IMC3_mud_list;
    IMC3_mud_list=new;
    new->name=str_dup("");
    new->ipaddress=str_dup("");
    new->mudlib=str_dup("");
    new->base_mudlib=str_dup("");
    new->driver=str_dup("");
    new->mud_type=str_dup("");
    new->open_status=str_dup("");
    new->admin_email=str_dup("");
    return new;
}

void destroy_imc3_mud(IMC3_MUD *mud) {
    if (mud==NULL) {
	bug("destroy_imc3_mud: Null parameter",0);
	return;
    }
    
    if (mud==IMC3_mud_list) {
	IMC3_mud_list=IMC3_mud_list->next;
    } else {
	IMC3_MUD *temp=IMC3_mud_list->next;
	while (temp!=mud && temp)
	    temp=temp->next;
	if (temp==NULL) {
	    bug("destroy_imc3_mud: Mud not found in list",0);
	    return;
	}
	temp->next=mud->next;
    }
    free_string(mud->name);
    free_string(mud->ipaddress);
    free_string(mud->mudlib);
    free_string(mud->base_mudlib);
    free_string(mud->driver);
    free_string(mud->mud_type);
    free_string(mud->open_status);
    free_string(mud->admin_email);
}

// listeners

IMC3_LISTENER *new_imc3_listener(IMC3_CHANNEL *channel) {
    IMC3_LISTENER *new;

    new=(IMC3_LISTENER *)malloc(sizeof(IMC3_LISTENER));
    bzero(new,sizeof(IMC3_LISTENER));
    new->next=channel->listener;
    channel->listener=new;
    return new;
}

void destroy_imc3_listener(IMC3_CHANNEL *channel,IMC3_LISTENER *listener) {
    if (channel==NULL) {
	bug("destroy_imc3_listener: Null parameter for channel",0);
	return;
    }
    if (listener==NULL) {
	bug("destroy_imc3_listener: Null parameter for listener",0);
	return;
    }
    
    if (channel->listener==listener) {
	channel->listener=channel->listener->next;
    } else {
	IMC3_LISTENER *temp=channel->listener;
	while (temp!=listener && temp)
	    temp=temp->next;
	if (temp==NULL) {
	    bug("destroy_imc3_listener: Listener not found in list",0);
	    return;
	}
	temp->next=listener->next;
    }
}

IMC3_LISTENER *find_imc3_listener_by_char(IMC3_CHANNEL *channel,CHAR_DATA *ch) {
    IMC3_LISTENER *this=channel->listener;

    while (this && this->ch!=ch)
	this=this->next;

    return this;

}
