// $Id: fd_imc3_rom.c,v 1.2 2000/02/17 09:00:18 edwin Exp $

/*
 * Copyright (c) 2000 Fatal Dimensions
 * 
 * See the file "LICENSE" or information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 * 
 */

#include <time.h>
#include <strings.h>
#include <stdio.h>
#include "../merc.h"
#include "../recycle.h"
#include "fd_imc3.h"

void imc3_mudlist(CHAR_DATA *ch,char *argument) {
    IMC3_MUD *mud;
    BUFFER *buf;
    char s[MSL],filter[MIL];
    bool all=FALSE;

    if (IS_NPC(ch)) return;

    argument=one_argument(argument,filter);
    if (str_cmp(filter,"all")==0) {
	all=TRUE;
	argument=one_argument(argument,filter);
    }

    buf=new_buf();
    add_buf(buf,"Name                      Type   Mudlib                  Address           Port\n\r");
    for (mud=IMC3_mud_list;mud;mud=mud->next) {
	if (filter[0] && str_prefix(filter,mud->name))
	    continue;
	if (!all && mud->status==0)		// do not print down
	    continue;

	switch (mud->status) {
	case -1:
	    sprintf(s,"%-26s%-7s%-24s%-17s %d\n\r",
		mud->name,mud->mud_type,mud->mudlib,mud->ipaddress,
		mud->player_port);
	    break;
	case 0:
	    sprintf(s,"%-26s(down)\n\r",mud->name);
	    break;
	default:
	    sprintf(s,"%-26s(rebooting, back in %d seconds)\n\r",
		mud->name,mud->status);
	    break;
	}
	add_buf(buf,s);
    }
    page_to_char(buf_string(buf),ch);
    free_buf(buf);
    return;
}

void imc3_chanlist(CHAR_DATA *ch,char *argument) {
    IMC3_CHANNEL *channel;
    BUFFER *buf;
    char s[MSL];
    bool all=FALSE;

    if (IS_NPC(ch)) return;

    buf=new_buf();
    channel=IMC3_channel_list;              
    add_buf(buf,"Local name          Lvl IMC3 Name           Hosted at\n\r");
    while (channel) {
	if (!all && channel->local_name[0]==0)
	    continue;
	sprintf(s,"%-20s%-4d%-20s%-20s\n\r",
	    channel->local_name,channel->local_level,
	    channel->imc3_name,channel->host_mud);
	add_buf(buf,s);
	channel=channel->next;
    }
    page_to_char(buf_string(buf),ch);
    free_buf(buf);
    return;
}

void imc3_stats(CHAR_DATA *ch,char *argument) {

    if (IS_NPC(ch)) return;

    sprintf_to_char(ch,"messages	: %d (%d unknown)\n",
					IMC3_stats.count_total,
					IMC3_stats.count_unknown);
    sprintf_to_char(ch,"private		: %d tells/%d emotes\n",
					IMC3_stats.count_tell,
					IMC3_stats.count_emoteto);
    sprintf_to_char(ch,"who		: %d commands/%d req/%d reply\n",
					IMC3_stats.count_who_commands,
					IMC3_stats.count_who_req,
					IMC3_stats.count_who_reply);
    sprintf_to_char(ch,"finger		: %d commands/%d req/%d reply\n",
					IMC3_stats.count_finger_commands,
					IMC3_stats.count_finger_req,
					IMC3_stats.count_finger_reply);
    sprintf_to_char(ch,"locate		: %d commands/%d req/%d reply\n",
					IMC3_stats.count_locate_commands,
					IMC3_stats.count_locate_req,
					IMC3_stats.count_locate_reply);
    sprintf_to_char(ch,"channels	: %d m/%d e/%d t sent\n",
					IMC3_stats.count_channel_m_commands,
					IMC3_stats.count_channel_e_commands,
					IMC3_stats.count_channel_t_commands);
    sprintf_to_char(ch,"		: %d m/%d e/%d t\n",
					IMC3_stats.count_channel_m,
					IMC3_stats.count_channel_e,
					IMC3_stats.count_channel_t);
    sprintf_to_char(ch,"		: %d list-reply/%d add/%d remove\n",
					IMC3_stats.count_chanlist_reply,
					IMC3_stats.count_channel_add,
					IMC3_stats.count_channel_remove);
    sprintf_to_char(ch,"		: filter %d req/%d reply\n",
					IMC3_stats.count_channel_filter_req,
					IMC3_stats.count_channel_filter_reply);
    sprintf_to_char(ch,"		: who %d commands/%d req/%d reply\n",
					IMC3_stats.count_channel_who_commands,
					IMC3_stats.count_channel_who_req,
					IMC3_stats.count_channel_who_reply);
    sprintf_to_char(ch,"		: user %d req/%d reply\n",
					IMC3_stats.count_chan_user_req,
					IMC3_stats.count_chan_user_reply);
    sprintf_to_char(ch,"news		: %d read/%d post/%d grplist_req\n",
					IMC3_stats.count_news_read_req,
					IMC3_stats.count_news_post_req,
					IMC3_stats.count_news_grplist_req);
    sprintf_to_char(ch,"mail		: %d/%d ack\n",
					IMC3_stats.count_mail,
					IMC3_stats.count_mail_ack);
    sprintf_to_char(ch,"filelist	: %d req/%d reply\n",
					IMC3_stats.count_file_list_req,
					IMC3_stats.count_file_list_reply);
    sprintf_to_char(ch,"file		: %d put/%d getreq/%d getreply\n",
					IMC3_stats.count_file_put,
					IMC3_stats.count_file_get_req,
					IMC3_stats.count_file_get_reply);
    sprintf_to_char(ch,"auth		: %d req/%d reply\n",
					IMC3_stats.count_auth_mud_req,
					IMC3_stats.count_auth_mud_reply);
    sprintf_to_char(ch,"startup		: %d req/%d reply\n",
					IMC3_stats.count_startup_req_3,
					IMC3_stats.count_startup_reply);
    sprintf_to_char(ch,"oob		: %d req/%d begin/%d end\n",
					IMC3_stats.count_oob_req,
					IMC3_stats.count_oob_begin,
					IMC3_stats.count_oob_end);
    sprintf_to_char(ch,"errors		: %d\n",
					IMC3_stats.count_error);
    sprintf_to_char(ch,"mudlist		: %d\n",
					IMC3_stats.count_mudlist);
    sprintf_to_char(ch,"shutdown	: %d\n",
					IMC3_stats.count_shutdown);
    sprintf_to_char(ch,"ucache		: %d\n",
					IMC3_stats.count_ucache_update);
}

void imc3_setup_channel(CHAR_DATA *ch,char *argument) {
    char localname[MIL];
    char imc3_name[MIL];
    char buf[MSL];
    IMC3_CHANNEL *channel;

    argument=one_argument(argument,imc3_name);
    argument=one_argument(argument,localname);

    if ((channel=find_imc3_channel_by_name(imc3_name))==NULL) {
	send_to_char("Unknown channel\n\r"
	    "(use {Wimc3 chanlist{x to get an overview of the channels available)\n\r",ch);
	return;
    }

    if (localname[0]==0) {
	free_string(channel->local_name);
	channel->local_name=str_dup("");
	sprintf(buf,"IMC3 - setup_channel: removing %s as %s @ %s",
	    localname,channel->imc3_name,channel->host_mud);
	wiznet(buf,NULL,NULL,WIZ_IMC3_INFO,0,0);
    } else {
	free_string(channel->local_name);
	channel->local_name=str_dup(localname);
	sprintf_to_char(ch,"%s @ %s is now locally known as %s\n\r",
	    channel->imc3_name,channel->host_mud,channel->local_name);
	sprintf(buf,"IMC3 - setup_channel: setting up to %s @ %s as %s",
	    channel->imc3_name,channel->host_mud,channel->local_name);
	wiznet(buf,NULL,NULL,WIZ_IMC3_INFO,0,0);
    }
}

void imc3_chan_who(CHAR_DATA *ch,char *argument) {
    char channel_name[MIL];
    char mud_name[MIL];
    IMC3_CHANNEL *channel;
    IMC3_MUD *mud;

    argument=one_argument(argument,channel_name);
    argument=one_argument(argument,mud_name);
    if ((channel=find_imc3_channel_by_localname(channel_name))==NULL) {
	send_to_char("Unknown channel.\n\r"
	    "(use {Wimc3 chanlist{x to get an overview of the channels available)\n\r",ch);
	return;
    }
    if ((mud=find_imc3_mud_by_name(mud_name))==NULL) {
	send_to_char("Unknown mud.\n\r"
	    "(use {Wimc3 mudlist{x to get an overview of the muds available)\n\r",
	    ch);
	return;
    }

    IMC3_send_chan_who(ch,channel,mud);
}

void imc3_listen_channel(CHAR_DATA *ch,char *argument) {
    char channel_name[MIL];
    IMC3_CHANNEL *channel;
    IMC3_LISTENER *listener;

    argument=one_argument(argument,channel_name);
    if ((channel=find_imc3_channel_by_localname(channel_name))==NULL) {
	send_to_char("Unknown channel.\n\r"
	    "(use {Wimc3 chanlist{x to get an overview of the channels available)\n\r",ch);
	return;
    }
    if ((listener=find_imc3_listener_by_char(channel,ch))!=NULL) {
	destroy_imc3_listener(channel,listener);
	sprintf_to_char(ch,"You are now removed from %s (%s @ %s)\n",
	    channel->imc3_name,channel->local_name,channel->host_mud);
	IMC3_check_channel(channel);
    } else {
	listener=new_imc3_listener(channel);
	listener->ch=ch;
	sprintf_to_char(ch,"You are now subscribed to %s (%s @ %s)\n",
	    channel->imc3_name,channel->local_name,channel->host_mud);
	IMC3_check_channel(channel);
    }
}

void do_imc3(CHAR_DATA *ch,char *argument) {
    char arg[MIL];

    argument=one_argument(argument,arg);
    switch (which_keyword(arg,"stats","setup","chanlist","mud","mudlist",
		"listen","chanwho",NULL)) {
	case 1: imc3_stats(ch,argument);break;
	case 2: imc3_setup_channel(ch,argument);break;
	case 3: imc3_chanlist(ch,argument);break;
	case 4:
	case 5: imc3_mudlist(ch,argument);break;
	case 6: imc3_listen_channel(ch,argument);break;
	case 7: imc3_chan_who(ch,argument);break;
	default: 
	    send_to_char("keywords expected:\n\r"
		"stats, chanlist [all] [filter], mudlist [filter],\n\r"
		"setup <channel> <localname>, listen <localname>,\n\r"
		"chanwho <channel>\n\r",ch);
    }
}


void do_rwho(CHAR_DATA *ch,char *argument) {
    IMC3_MUD *mud;

    if (IS_NPC(ch)) return;

    if (argument[0]==0) {
	send_to_char("Get an overview of which mud?\n\r"
	    "(use {Wimc3 mudlist{x to get an overview of the muds available)\n\r",
	    ch);
	return;
    }

    if ((mud=find_imc3_mud_by_name(argument))==NULL) {
	send_to_char("No such mud known.\n\r"
	    "(use {Wimc3 mudlist{x to get an overview of the muds available)\n\r",
	    ch);
	return;
    }
    IMC3_send_who(ch,mud->name);
}

void do_rlocate(CHAR_DATA *ch,char *argument) {

    if (IS_NPC(ch)) return;

    if (argument[0]==0) {
	send_to_char("Locate who?\n\r",ch);
	return;
    }
    IMC3_send_locate(ch,argument);
}

void do_rfinger(CHAR_DATA *ch,char *argument) {
    char user[MSL],mud[MSL];
    char *ps;
    IMC3_MUD *pmud;

    if (IS_NPC(ch)) return;

    if (argument[0]==0) {
	send_to_char("Finger who at which mud?\n\r"
	    "(use {Wimc3 mudlist{x to get an overview of the muds available)\n\r",
	    ch);
	return;
    }
    if ((ps=strchr(argument,'@'))==NULL) {
	send_to_char("You should specify a person and a mud.\n\r"
	    "(use {Wimc3 mudlist{x to get an overview of the muds available)\n\r",
	    ch);
	return;
    }

    ps[0]=0;
    strcpy(user,argument);
    strcpy(mud,ps+1);

    if (user[0]==0 || mud[0]==0) {
	send_to_char("You should specify a person and a mud.\n\r"
	    "(use {Wimc3 mudlist{x to get an overview of the muds available)\n\r",
	    ch);
	return;
    }

    if ((pmud=find_imc3_mud_by_name(mud))==NULL) {
	send_to_char("No such mud known.\n\r"
	    "(use {Wimc3 mudlist{x to get an overview of the muds available)\n\r",
	    ch);
	return;
    }

    IMC3_send_finger(ch,user,pmud->name);
}

