// $Id: fd_imc3_support.c,v 1.4 2000/02/17 09:00:19 edwin Exp $

/*
 * Copyright (c) 2000 Fatal Dimensions
 * 
 * See the file "LICENSE" or information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 * 
 */

#include <stdio.h>
#include <strings.h>
#include <stdlib.h>
#include <time.h>
#include "../merc.h"
#include "fd_imc3.h"


void IMC3_startup_packet(void) {

    IMC3_stats.count_startup_req_3++;

    IMC3_write_header("startup-req-3",IMC3_THISMUD,NULL,IMC3_ROUTER_NAME,0);
    
    IMC3_write_buffer("2286,523060,57,4000,0,0,");
    IMC3_write_buffer(IMC3_MUDLIB);
    IMC3_write_buffer(",");
    IMC3_write_buffer(IMC3_BASE_MUDLIB);
    IMC3_write_buffer(",");
    IMC3_write_buffer(IMC3_DRIVER);
    IMC3_write_buffer(",");
    IMC3_write_buffer(IMC3_MUD_TYPE);
    IMC3_write_buffer(",");
    IMC3_write_buffer(IMC3_OPEN_STATUS);
    IMC3_write_buffer(",");
    IMC3_write_buffer(IMC3_ADMIN_EMAIL);
    IMC3_write_buffer(",([\"emoteto\":1,\"news\":0,\"ucache\":0,\"auth\":0,\"ftp\":0,\"tell\":0,\"mail\":0,\"file\":0,\"http\":4001,\"locate\":0,\"finger\":0,\"channel\":0,\"who\":0,]),0,})\r");

    IMC3_send_buffer();
}


/*
startup-reply
   
   This packet will be delivered to a mud for three conditions: in
   response to a startup-req packet, when the router wishes the mud to
   connect to a different router, or when the set of routers change for
   some reason.                      
    ({
        (string)   "startup-reply",
        (int)      5, 
        (string)   originator_mudname,     // the router
        (string)   0,
        (string)   target_mudname,
        (string)   0,
        (string *) router_list,
        (int)      password
    })                        

   The router_list is an array representing an ordered list of routers to
   use. The first element should be the router that the mud should use.
   Typically, this will be the router that the mud initially connected
   to. If not, however, then the mud should close the connection and
   reopen to the designated router. The list should be saved and used in
   case of failure to connect to a router. Each element in the list is an
   array of two elements; the first element is the router name, the   
   second element is the router's address in the following format:
   "ip.ad.re.ss portnum". Note that this address can be passed to MudOS's
   socket_connect() function. For example: ({ "*nightmare",     
   "199.199.122.10 9000" }).                                       
   
   The first router specified in the list will be the mud's preferred   
   router. Future initial connections and startup-req packets will go to
   that router until told otherwise.

*/

int IMC3_process_startup_reply(char *s) {
    int i;
    char *ps=s,*next_ps;
    IMC3_HEADER header;

    IMC3_get_header(&ps,&header);

    for (i=0;i<2;i++) {
	IMC3_get_field(ps,&next_ps);
//	printf("Got field %d: .%s.\n",i,ps);
	ps=next_ps;
    }
    return 0;
}   



/*
   The router will send this to a mud whenever the mud's list needs to be
   updated. Typically, this will happen once right after login (based on 
   the old_mudlist_id that a mud provided in the startup-req-3 packet),
   and then as changes occur within the intermud network. A mud should 
   remember the mudlist and its associated mudlist_id across reconnects
   to the router.                                                       
    ({                                                                   
        (string)  "mudlist",                                          
        (int)     5,
        (string)  originator_mudname,     // the router             
        (string)  0,                                                  
        (string)  target_mudname,
        (string)  0,       
        (int)     mudlist_id  
        (mapping) info_mapping                                      
    })  

   The info_mapping contains mud names as keys and information about each
   mud as the value. This information is specified as an array with the
   following format:                                                 
    ({                                                                
        (int)     state,                                               
        (string)  ip_addr,                                              
        (int)     player_port,                                           
        (int)     imud_tcp_port,                                         
        (int)     imud_udp_port,                                      
        (string)  mudlib,
        (string)  base_mudlib,
        (string)  driver,                                               
        (string)  mud_type,                      
        (string)  open_status,
        (string)  admin_email,                                        
        (mapping) services                                              
        (mapping) other_data                                           
    })           
     
   Each record of information should replace any prior record for a    
   particular mud. If the mapping's value is zero, then the mud has been
   deleted (it went down and has not come back for a week) from the    
   Intermud.           

   state is an integer with the following values:                   
        -1  mud is up                                                    
         0  mud is down                                                
         n  mud will be down for n seconds                        
*/

int IMC3_process_mudlist(char *s) {
    char *ps=s,*next_ps;
    IMC3_MUD *mud;
    IMC3_HEADER header;

    IMC3_get_header(&ps,&header);
    IMC3_get_field(ps,&next_ps);	// password
    ps=next_ps;
    ps+=2;

    // info_mapping
    while (1) {
	char *next_ps2;
	// first the name
	IMC3_get_field(ps,&next_ps);
	IMC3_remove_quotes(&ps);
	mud=find_imc3_mud_by_name(ps);
	if (mud==NULL) {
	    mud=new_imc3_mud();
	    free_string(mud->name);mud->name=str_dup(ps);
	}
//	printf("- name: %s\n",mud->name);

	ps=next_ps;
	IMC3_get_field(ps,&next_ps2);	// next_ps2 is the end of the services
//	printf("  %s\n",ps);

	if (ps[0]!='0') {
	    ps+=2;

	    IMC3_get_field(ps,&next_ps);
	    mud->status=atoi(ps);
	    ps=next_ps;

	    IMC3_get_field(ps,&next_ps);
	    IMC3_remove_quotes(&ps);
	    free_string(mud->ipaddress);mud->ipaddress=str_dup(ps);
	    ps=next_ps;
//	    printf("  ipaddress: %s\n",mud->ipaddress);

	    IMC3_get_field(ps,&next_ps);
	    mud->player_port=atoi(ps);
	    ps=next_ps;

	    IMC3_get_field(ps,&next_ps);
	    mud->imud_tcp_port=atoi(ps);
	    ps=next_ps;

	    IMC3_get_field(ps,&next_ps);
	    mud->imud_udp_port=atoi(ps);
	    ps=next_ps;

	    IMC3_get_field(ps,&next_ps);
	    IMC3_remove_quotes(&ps);
	    free_string(mud->mudlib);mud->mudlib=str_dup(ps);
	    ps=next_ps;
//	    printf("  mudlib: %s\n",mud->mudlib);

	    IMC3_get_field(ps,&next_ps);
	    IMC3_remove_quotes(&ps);
	    free_string(mud->base_mudlib);mud->base_mudlib=str_dup(ps);
	    ps=next_ps;
//	    printf("  base_mudlib: %s\n",mud->base_mudlib);

	    IMC3_get_field(ps,&next_ps);
	    IMC3_remove_quotes(&ps);
	    free_string(mud->driver);mud->driver=str_dup(ps);
	    ps=next_ps;
//	    printf("  driver: %s\n",mud->driver);

	    IMC3_get_field(ps,&next_ps);
	    IMC3_remove_quotes(&ps);
	    free_string(mud->mud_type);mud->mud_type=str_dup(ps);
	    ps=next_ps;
//	    printf("  type: %s\n",mud->mud_type);

	    IMC3_get_field(ps,&next_ps);
	    IMC3_remove_quotes(&ps);
	    free_string(mud->open_status);mud->open_status=str_dup(ps);
	    ps=next_ps;
//	    printf("  status: %s\n",mud->open_status);

	    IMC3_get_field(ps,&next_ps);
	    IMC3_remove_quotes(&ps);
	    free_string(mud->admin_email);mud->admin_email=str_dup(ps);
	    ps=next_ps;
//	    printf("  admin: %s\n",mud->admin_email);

	    IMC3_get_field(ps,&next_ps);
//	    printf("  junk: %s\n",ps);
	    ps=next_ps;

	    IMC3_get_field(ps,&next_ps);
//	    printf("  services: %s\n",ps);
	    ps=next_ps;

	}
	ps=next_ps2;
	if (ps[0]==']')
	    break;
    }


    return 0;
}   


/*
({"error",5,"FF-Test",0,"NoNameMUD","mavetju","unk-type","type 'locate-req' is unrecognized",({"locate-req",5,"NoNameMUD","mavetju",0,0,"blaat",}),}).
*/
void IMC3_process_error(char *s) {
    IMC3_HEADER header;
    char *next_ps,*ps=s;
    char type[MSL],message[MSL],error[MSL];

    IMC3_get_header(&ps,&header);

    IMC3_get_field(ps,&next_ps);
    IMC3_remove_quotes(&ps);
    strcpy(type,ps);
    ps=next_ps;

    IMC3_get_field(ps,&next_ps);
    IMC3_remove_quotes(&ps);
    strcpy(message,ps);

    sprintf(error,"IMC3 - Error: from %s to %s @ %s (%s) %s",
	header.originator_mudname,
	header.target_username,header.target_mudname,
	type,message);
    wiznet(error,NULL,NULL,WIZ_IMC3_ERRORS,0,0);
}
