// $Id: fd_imc3_who.c,v 1.4 2000/02/17 09:00:19 edwin Exp $

/*
 * Copyright (c) 2000 Fatal Dimensions
 * 
 * See the file "LICENSE" or information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 * 
 */

#include <stdio.h>
#include <strings.h>
#include <time.h>
#include "../merc.h"
#include "fd_imc3.h"

int IMC3_send_who(CHAR_DATA *ch,char *mud) {

    IMC3_stats.count_who_commands++;

    IMC3_escape(mud);
    IMC3_write_header("who-req",IMC3_THISMUD,ch->name,mud,NULL);
    IMC3_write_buffer("})\r");
    IMC3_send_buffer();
    return 0;
}

/*
   The router will route the packet to another router or to the target
   mud. The target mud returns:
    ({
        (string)  "who-reply",                                         
        (int)     5,                                                   
        (string)  originator_mudname,                                 
        (string)  0,                                  
        (string)  target_mudname,
        (string)  target_username,                                  
        (mixed *) who_data
    })                                                               
   
   where who_data is an array containing an array of the following format
   for each user on the mud:
    ({
        (string)  user_visname,                                      
        (int)     idle_time,                
        (string)  xtra_info          
    })                     

   Each user_visname should specify the user's visual name. idle_time
   should be measured in seconds and xtra_info should be a string.
 */
int IMC3_process_who_reply(char *s) {
    char *ps=s,*next_ps,*next_ps2; 
    CHAR_DATA *ch;
    IMC3_HEADER header;

    IMC3_get_header(&ps,&header);

    if ((ch=IMC3_find_user(header.target_username))==NULL) {
	char s[MSL];

	sprintf(s,"IMC3 - IMC3_process_who_reply(): user %s not found.",
	    header.target_username);
	bug(s,0);
	return 0;
    }

    ps+=2;

// ({"who-reply",5,"NoNameMUD",0,"MavEtJu","MavEtJu",({({"Mavetju",8,"Mavetju the Newbie",}),}),})
    sprintf_to_char(ch,"RWHO reply from %s\n\r",header.originator_mudname);
    while (1) {
	if (ps[0]=='}') {
	    send_to_char("No information returned.\n\r",ch);
	    return 0;
	}

	IMC3_get_field(ps,&next_ps);

	ps+=2;
	IMC3_get_field(ps,&next_ps2);
	IMC3_remove_quotes(&ps);
	sprintf_to_char(ch,"%-15s",ps);
	ps=next_ps2;
	IMC3_get_field(ps,&next_ps2);
	sprintf_to_char(ch,"(idle %s secs)%s",
	    ps,strlen(ps)==1?"   ":strlen(ps)==2?"  ":strlen(ps)==3?" ":"");
	ps=next_ps2;
	IMC3_get_field(ps,&next_ps2);
	IMC3_remove_quotes(&ps);
	sprintf_to_char(ch," %s\n\r",ps);
	ps=next_ps2;

	ps=next_ps;
	if (ps[0]=='}')
	    break;
    }

    return 0;
}

int IMC3_process_who_req(char *s) {
    char *ps=s;
    IMC3_HEADER header;
    DESCRIPTOR_DATA *d;
    char buf[MSL],smallbuf[MSL];
#ifdef IS_FD
    time_t now=time(NULL);
#endif

    IMC3_get_header(&ps,&header);

    sprintf(buf,"IMC3 - Who: request from %s @ %s.",
	header.originator_username,header.originator_mudname);
    wiznet(buf,NULL,NULL,WIZ_IMC3_INFO,0,0);

    IMC3_write_header("who-reply",IMC3_THISMUD,NULL,
	header.originator_mudname,header.originator_username);
    IMC3_write_buffer("({");

    for (d=descriptor_list;d;d=d->next) {
	if (d->character &&
	    d->connected==CON_PLAYING) {
	    IMC3_write_buffer("({\"");
	    IMC3_write_buffer(IMC3_escape(d->character->name));
	    IMC3_write_buffer("\",");
#ifdef IS_FD
	    sprintf(smallbuf,"%ld",now-d->character->pcdata->idle);
#else
	    sprintf(smallbuf,"%ld",-1l);
#endif
	    IMC3_write_buffer(smallbuf);
	    IMC3_write_buffer(",\"");
	    IMC3_write_buffer(IMC3_escape(d->character->name));
	    IMC3_write_buffer(IMC3_escape(d->character->pcdata->title));
	    IMC3_write_buffer("\",}),");
	}
    }
    IMC3_write_buffer("}),})\r");
    IMC3_send_buffer();

    return 0;
}
